
"""
%prog

Michael Coughlin (coughlim@carleton.edu)

This program runs seismon_run.py.

"""

# =============================================================================
#
#                               PREAMBLE
#
# =============================================================================


import os, time
import numpy as np

__author__ = "Michael Coughlin <coughlim@carleton.edu>"
__date__ = "2012/2/7"
__version__ = "0.1"

gpsStart = 1009411215
numFrames = 24*366
#numFrames = 25*1
frameDur = 3600
gpsEnd = 1009411215 + (numFrames+1)*frameDur

gpss = np.arange(gpsStart,gpsEnd,frameDur)
gpsStart = 189331200
numFrames = 24*366

framesFolder = "/home/mcoughlin/Stochastic/Moon/frames/"
paramsFile = "/home/mcoughlin/Seismon/seismon/input/seismon_params_IRIS.txt"

for i in xrange(len(gpss)-1):
   frameFile = "%s/IRIS-IRIS-%d-%d.gwf"%(framesFolder,gpss[i],gpss[i+1]-gpss[i])

   if not os.path.isfile(frameFile):
       os.system("python seismon_run -p %s -s %d -e %d --doMakeFrames --doPlots --framesSampleRate 8"%(paramsFile,gpss[i],gpss[i+1]))

#for i in [1000]:
#for i in [5000]:
#   frameFile = "%s/IRIS-IRIS-%d-%d.gwf"%(framesFolder,gpss[i],gpss[i+1]-gpss[i])

#   os.system("python seismon_run -p %s -s %d -e %d --doMakeFramesCalibrated --doPlots --framesSampleRate 8"%(paramsFile,gpss[i],gpss[i+1]))




#!/usr/bin/python

import sys, os, glob
import numpy as np
import scipy.linalg

import seismon.utils

try:
    import gwpy.time, gwpy.timeseries
    import gwpy.frequencyseries, gwpy.spectrogram
except:
    print("gwpy import fails... no plotting possible.")

__author__ = "Michael Coughlin <michael.coughlin@ligo.org>"
__date__ = "2012/8/26"
__version__ = "0.1"

# =============================================================================
#
#                               DEFINITIONS
#
# =============================================================================

def wiener(params, target_channel, segment):
    """@calculates wiener filter for given channel and segment.

    @param params
        seismon params dictionary
    @param target_channel
        seismon channel structure
    @param segment
        [start,end] gps
    """

    ifo = seismon.utils.getIfo(params)

    gpsStart = segment[0]
    gpsEnd = segment[1]

    # set the times
    duration = np.ceil(gpsEnd-gpsStart)

    dataAll = []

    if params["wienerFilterSampleRate"] > 0:
        samplef =params["wienerFilterSampleRate"]
    else:
        samplef = target_channel.samplef
    N = params["wienerFilterOrder"]

    for channel in params["channels"]:
        # make timeseries
        dataFull = seismon.utils.retrieve_timeseries(params, channel, segment)
        if dataFull == []:
            continue

        dataFull = dataFull / channel.calibration
        indexes = np.where(np.isnan(dataFull.data))[0]
        meanSamples = np.mean(np.ma.masked_array(dataFull.data,np.isnan(dataFull.data)))
        for index in indexes:
            dataFull[index] = meanSamples
        dataFull -= np.mean(dataFull.data)

        if np.mean(dataFull.data) == 0.0:
            print("data only zeroes... continuing\n")
            continue
        if len(dataFull.data) < 2*channel.samplef:
            print("timeseries too short for analysis... continuing\n")
            continue

        if params["wienerFilterSampleRate"] > 0:
            dataFull = dataFull.resample(params["wienerSampleRate"])

        if (params["wienerFilterLowFreq"] > 0) and (params["wienerFilterHighFreq"] == 0):
            dataFull = dataFull.highpass(params["wienerFilterLowFreq"], amplitude=0.9, order=3, method='scipy')
        elif (params["wienerFilterLowFreq"] == 0) and (params["wienerFilterHighFreq"] > 0):
            dataFull = dataFull.lowpass(params["wienerFilterHighFreq"], amplitude=0.9, order=3, method='scipy')
        elif (params["wienerFilterLowFreq"] > 0) and (params["wienerFilterHighFreq"] > 0):
            dataFull = dataFull.bandpass(params["wienerFilterLowFreq"],params["wienerFilterHighFreq"], amplitude=0.9, order=3, method='scipy')

        indexes = np.where(np.isnan(dataFull.data))[0]
        meanSamples = np.mean(np.ma.masked_array(dataFull.data,np.isnan(dataFull.data)))
        for index in indexes:
            dataFull[index] = meanSamples
        dataFull -= np.mean(dataFull.data)

        dataAll.append(dataFull)

    X = []
    y = []
    for dataFull in dataAll:
        if dataFull.channel.name == target_channel.station:
            tt = np.array(dataFull.times)
            y = dataFull.fftgram(params["fftDuration"])
        else:
            x = dataFull.fftgram(params["fftDuration"]) 
            X.append(x)

    if len(y) == 0:
        print("No data for target channel... continuing")
        return

    gpss = y.times

    originalASD = []
    residualASD = []
    FFASD = []

    create_filter = True
    for i in range(len(gpss)):

        if create_filter:
            indexes = np.arange(i,i+N)
        elif i<N:
            continue
        else:
            indexes = np.arange(i,i+1)

        yCut = y[indexes]
        XCut = []
        for x in X:
            XCut.append(x[indexes])

        if create_filter:
            print("Generating filter")
            W,R,P = miso_firwiener_fft(N,XCut,yCut)
            create_filter = False
            print("finished generating filter")
            continue

        residual, FF = subtractFF(W,XCut,yCut,samplef)
       
        yCut = np.fft.ifft(yCut.data[0]).real
        residual = np.fft.ifft(residual).real
        FF = np.fft.ifft(FF).real

        thisGPSStart = gpss[i]
        dataOriginal = gwpy.timeseries.TimeSeries(yCut, epoch=thisGPSStart, sample_rate=samplef,name="Original")
        dataResidual = gwpy.timeseries.TimeSeries(residual, epoch=thisGPSStart, sample_rate=samplef,name="Residual")
        dataFF = gwpy.timeseries.TimeSeries(FF, epoch=thisGPSStart, sample_rate=samplef,name="FF")

        # calculate spectrum
        NFFT = params["fftDuration"]
        #window = None
        dataOriginalASD = dataOriginal.asd(NFFT,NFFT,'welch')
        dataResidualASD = dataResidual.asd(NFFT,NFFT,'welch')
        dataFFASD = dataFF.asd(NFFT,NFFT,'welch')

        freq = np.array(dataOriginalASD.frequencies)
        indexes = np.where((freq >= params["fmin"]) & (freq <= params["fmax"]))[0]
        freq = freq[indexes]

        dataOriginalASD = np.array(dataOriginalASD.data)
        dataOriginalASD = dataOriginalASD[indexes]
        dataOriginalASD = gwpy.frequencyseries.Spectrum(dataOriginalASD, f0=np.min(freq), df=(freq[1]-freq[0]))

        dataResidualASD = np.array(dataResidualASD.data)
        dataResidualASD = dataResidualASD[indexes]
        dataResidualASD = gwpy.frequencyseries.Spectrum(dataResidualASD, f0=np.min(freq), df=(freq[1]-freq[0]))

        dataFFASD = np.array(dataFFASD.data)
        dataFFASD = dataFFASD[indexes]
        dataFFASD = gwpy.frequencyseries.Spectrum(dataFFASD, f0=np.min(freq), df=(freq[1]-freq[0]))

        originalASD.append(dataOriginalASD)
        residualASD.append(dataResidualASD)
        FFASD.append(dataFFASD)

    dt = gpss[1] - gpss[0]
    epoch = gwpy.time.Time(gpss[0], format='gps')
    originalSpecgram = gwpy.spectrogram.Spectrogram.from_spectra(*originalASD, dt=dt,epoch=epoch)
    residualSpecgram = gwpy.spectrogram.Spectrogram.from_spectra(*residualASD, dt=dt,epoch=epoch)
    FFSpecgram = gwpy.spectrogram.Spectrogram.from_spectra(*FFASD, dt=dt,epoch=epoch)

    freq = np.array(originalSpecgram.frequencies)

    kwargs = {'log':True,'nbins':500,'norm':True}
    originalSpecvar = gwpy.frequencyseries.hist.SpectralVariance.from_spectrogram(originalSpecgram,**kwargs)
    bins = originalSpecvar.bins[:-1]
    originalSpecvar = originalSpecvar * 100
    original_spectral_variation_50per = originalSpecvar.percentile(50)
    residualSpecvar = gwpy.frequencyseries.hist.SpectralVariance.from_spectrogram(residualSpecgram,**kwargs)
    bins = residualSpecvar.bins[:-1]
    residualSpecvar = residualSpecvar * 100
    residual_spectral_variation_50per = residualSpecvar.percentile(50)
    FFSpecvar = gwpy.frequencyseries.hist.SpectralVariance.from_spectrogram(FFSpecgram,**kwargs)
    bins = FFSpecvar.bins[:-1]
    FFSpecvar = FFSpecvar * 100
    FF_spectral_variation_50per = FFSpecvar.percentile(50)

    psdDirectory = params["dirPath"] + "/Text_Files/WienerFFT/" + target_channel.station_underscore + "/" + str(params["fftDuration"]) + "/" + str(params["wienerFilterOrder"])
    seismon.utils.mkdir(psdDirectory)

    freq = np.array(residual_spectral_variation_50per.frequencies)

    psdFile = os.path.join(psdDirectory,"%d-%d.txt"%(gpsStart,gpsEnd))
    f = open(psdFile,"wb")
    for i in range(len(freq)):
        f.write("%e %e\n"%(freq[i],residual_spectral_variation_50per[i]))
    f.close()

    if params["doPlots"]:

        plotDirectory = params["path"] + "/WienerFFT/" + target_channel.station_underscore + "/" + str(params["fftDuration"]) + "/" + str(params["wienerFilterOrder"])
        seismon.utils.mkdir(plotDirectory)

        fl, low, fh, high = seismon.NLNM.NLNM(2)

        pngFile = os.path.join(plotDirectory,"psd.png")

        plot = gwpy.plotter.Plot(figsize=[14,8])
        kwargs = {"linestyle":"-","color":"b","label":"Original"}
        plot.add_line(freq, original_spectral_variation_50per, **kwargs)
        kwargs = {"linestyle":"-","color":"g","label":"Residual"}
        plot.add_line(freq, residual_spectral_variation_50per, **kwargs)
        kwargs = {"linestyle":"-","color":"r","label":"FF"}
        plot.add_line(freq, FF_spectral_variation_50per, **kwargs)
        kwargs = {"linestyle":"-.","color":"k"}
        plot.add_line(fl, low, **kwargs)
        plot.add_line(fh, high, **kwargs)
        plot.axes[0].set_yscale("log")
        plot.axes[0].set_xscale("log")
        plot.xlim = [params["fmin"],params["fmax"]]
        #plot.ylim = [np.min(bins), np.max(bins)]
        plot.xlabel = "Frequency [Hz]"
        plot.ylabel = "Amplitude Spectrum [(m/s)/rtHz]"
        plot.add_legend(loc=1,prop={'size':10})
        plot.save(pngFile,dpi=200)
        plot.close()

def miso_firwiener_fft(N,X,y):

    # MISO_FIRWIENER Optimal FIR Wiener filter for multiple inputs.
    # MISO_FIRWIENER(N,X,Y) computes the optimal FIR Wiener filter of order
    # N, given any number of (stationary) random input signals as the columns
    # of matrix X, and one output signal in column vector Y.
    # Author: Keenan Pepper
    # Last modified: 2007/08/02
    # References:
    # [1] Y. Huang, J. Benesty, and J. Chen, Acoustic MIMO Signal
    # Processing, SpringerVerlag, 2006, page 48

    # Number of input channels.

    M = len(X)

    freqs = np.array(y.frequencies)

    R = np.zeros([M,M,len(freqs)])
    P = np.zeros([M,len(freqs)])
    W = np.zeros([M,len(freqs)])

    for i in range(len(freqs)):
        yCut = y.data[:,i]
        XCut = []
        for x in X:
            XCut.append(x.data[:,i])

        for j in range(M):
            for k in range(M):

                a1 = XCut[j]
                psd1 = np.mean(a1 * np.conjugate(a1)).real
                a2 = XCut[k]
                psd2 = np.mean(a2 * np.conjugate(a2)).real
                csd12 = np.mean(a1 * np.conjugate(a2))
                coh = np.absolute(csd12) / np.sqrt(psd1 * psd2)

                R[j,k,i] = csd12

            a1 = XCut[j]
            psd1 = np.mean(a1 * np.conjugate(a1)).real
            a2 = yCut
            psd2 = np.mean(a2 * np.conjugate(a2)).real
            csd12 = np.mean(a1 * np.conjugate(a2))
            coh = np.absolute(csd12) / np.sqrt(psd1 * psd2)

            P[j,i] = csd12
  
    for i in range(len(freqs)):
        Rinv = scipy.linalg.inv(R[1:,1:,i])
        c = P[1:,i]

        W[:,i] = Rinv.dot(c)
 
    return W,R,P

def subtractFF(W,SS,S,samplef):

    # Subtracts the filtered SS from S using FIR filter coefficients W.
    # Routine written by Jan Harms. Routine modified by Michael Coughlin.
    # Modified: August 17, 2012
    # Contact: michael.coughlin@ligo.org

    M = len(SS)

    freqs = np.array(S.frequencies)

    residual = []
    FF = []

    for i in range(len(freqs)):
        Wtemp = W[:,i]
        Xtemp = []
        for x in SS:
            Xtemp.append(x.data[0,i])
        FF.append(np.sum(Wtemp*Xtemp))

    residual = S.data[0] - np.array(FF)

    return residual, FF

def wiener_summary(params, target_channel, segment):
    """@calculates wiener filter for given channel and segment.

    @param params
        seismon params dictionary
    @param target_channel
        seismon channel structure
    @param segment
        [start,end] gps
    """

    gpsStart = segment[0]
    gpsEnd = segment[1]

    psdDirectory = params["dirPath"] + "/Text_Files/WienerFFT/" + target_channel.station_underscore + "/" + str(params["fftDuration"])

    directories = glob.glob(os.path.join(psdDirectory,"*"))

    data = {}

    for directory in directories:

        directorySplit = directory.split("/")
        N = int(directorySplit[-1])

        file = os.path.join(directory,"%d-%d.txt"%(gpsStart,gpsEnd))

        if not os.path.isfile(file):
            continue

        spectra_out = gwpy.frequencyseries.Spectrum.read(file)
        spectra_out.unit = 'counts/Hz^(1/2)'

        if np.sum(spectra_out.data) == 0.0:
            continue

        data[str(N)] = {}
        data[str(N)]["data"] = spectra_out

    if data == {}:
        return

    if params["doPlots"]:

        plotDirectory = params["path"] + "/wiener_summary/" + target_channel.station_underscore
        seismon.utils.mkdir(plotDirectory)

        fl, low, fh, high = seismon.NLNM.NLNM(2)

        pngFile = os.path.join(plotDirectory,"psd.png")
        lowBin = np.inf
        highBin = -np.inf
        plot = gwpy.plotter.Plot(figsize=[14,8])
        for key in data.iterkeys():

            label = key.replace("_","\_")

            plot.add_spectrum(data[key]["data"], label=label)
            lowBin = np.min([lowBin,np.min(data[key]["data"])])
            highBin = np.max([highBin,np.max(data[key]["data"])])

        kwargs = {"linestyle":"-.","color":"k"}
        plot.add_line(fl, low, **kwargs)
        plot.add_line(fh, high, **kwargs)
        plot.xlim = [params["fmin"],params["fmax"]]
        plot.ylim = [lowBin, highBin]
        plot.xlabel = "Frequency [Hz]"
        plot.ylabel = "Amplitude Spectrum [(m/s)/rtHz]"
        plot.add_legend(loc=1,prop={'size':10})
        plot.axes[0].set_xscale("log")
        plot.axes[0].set_yscale("log")

        plot.save(pngFile,dpi=200)
        plot.close()





#!D:\bld\seismon_1650995782590\_h_env\python.exe

# EPICS IOC for seismon data
# K. Thorne - use stdenv, allow parameter for data directory

# Setup up CDS standard environment
import sys
sys.path.append('/ligo/cdscfg')
import stdenv as cds
cds.INIT_ENV()

print "Run this for site " + cds.SITE + " ifo " + cds.IFO

# create server
from pcaspy import Driver, SimpleServer
import time

prefix = cds.IFO+':CDS-SEISMON_'
pvdb = {
    'EQ_TIME_GPS' : {
        'prec' : 3,
    },
    'EQ_TIME_UTC' : {
        'type' : 'string',
    },
    'EQ_TIME_LTZ' : {
        'type' : 'string',
    },
    'EQ_MAGNITUDE_MMS' : {
        'prec' : 3,
    },
    'EQ_LATITUDE_DEG' : {
        'prec' : 3,
    },
    'EQ_LONGITUDE_DEG' : {
        'prec' : 3,
    },
    'EQ_DEPTH_KM' : {
        'prec' : 3,
    },
    'EQ_' + cds.SITE + '_DISTANCE_M' : {
        'prec' : 3,
    },
    cds.SITE + '_P_ARRIVALTIME_GPS' : {
        'prec' : 3,
    },
    cds.SITE + '_S_ARRIVALTIME_GPS' : {
        'prec' : 3,
    },
    cds.SITE + '_R20_ARRIVALTIME_GPS' : {
        'prec' : 3,
    },
    cds.SITE + '_R35_ARRIVALTIME_GPS' : {
        'prec' : 3,
    },
    cds.SITE + '_R50_ARRIVALTIME_GPS' : {
        'prec' : 3,
    },
    cds.SITE + '_P_VELOCITY_MPS' : {
        'prec' : 3,
    },
    cds.SITE + '_S_VELOCITY_MPS' : {
        'prec' : 3,
    },
    cds.SITE + '_R20_VELOCITY_MPS' : {
        'prec' : 3,
    },
    cds.SITE + '_R35_VELOCITY_MPS' : {
        'prec' : 3,
    },
    cds.SITE + '_R50_VELOCITY_MPS' : {
        'prec' : 3,
    },
    cds.SITE + '_P_ARRIVALTIME_MINS' : {
        'prec' : 0,
    },
    cds.SITE + '_P_ARRIVALTIME_SECS' : {
        'prec' : 0,
    },
    cds.SITE + '_P_ARRIVALTIME_TOTAL_SECS' : {
        'prec' : 0,
    },
    cds.SITE + '_S_ARRIVALTIME_MINS' : {
        'prec' : 0,
    },
    cds.SITE + '_S_ARRIVALTIME_SECS' : {
        'prec' : 0,
    },
    cds.SITE + '_S_ARRIVALTIME_TOTAL_SECS' : {
        'prec' : 0,
    },
    cds.SITE + '_R20_ARRIVALTIME_MINS' : {
        'prec' : 0,
    },
    cds.SITE + '_R20_ARRIVALTIME_SECS' : {
        'prec' : 0,
    },
    cds.SITE + '_R20_ARRIVALTIME_TOTAL_SECS' : {
        'prec' : 0,
    },
    cds.SITE + '_R35_ARRIVALTIME_MINS' : {
        'prec' : 0,
    },
    cds.SITE + '_R35_ARRIVALTIME_SECS' : {
        'prec' : 0,
    },
    cds.SITE + '_R35_ARRIVALTIME_TOTAL_SECS' : {
        'prec' : 0,
    },
    cds.SITE + '_R50_ARRIVALTIME_MINS' : {
        'prec' : 0,
    },
    cds.SITE + '_R50_ARRIVALTIME_SECS' : {
        'prec' : 0,
    },
    cds.SITE + '_R50_ARRIVALTIME_TOTAL_SECS' : {
        'prec' : 0,
    },
    'SYSTEM_TIME_GPS' : {
        'prec' : 0,
    },
    'SYSTEM_UPTIME_SEC' : {
        'prec' : 0,
    },
    'SYSTEM_COUNTER' : {
        'prec' : 0,
    },
}

ioc_uptime = 0

class myDriver(Driver):
    def  __init__(self):
        super(myDriver, self).__init__()

if __name__ == '__main__':
    iocuptime = 0
    server = SimpleServer()
    server.createPV(prefix, pvdb)
    driver = myDriver()

    # process CA transactions
    while True:
        server.process(0.1)
	ioc_uptime = ioc_uptime + 1





# Copyright (C) 2013 Michael Coughlin
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""Earthquake xml file generator.

This script generates earthquake xml files using notices from the
internet and USGS PDL client.

Comments should be e-mailed to michael.coughlin@ligo.org.

"""

import os, sys, glob, optparse, warnings, time, json
import numpy as np
import subprocess
from subprocess import Popen
from lxml import etree

import lal.gpstime

from seismon import (eqmon, utils)

import matplotlib
matplotlib.use("AGG")
matplotlib.rcParams.update({'font.size': 18})
from matplotlib import pyplot as plt
from matplotlib import cm

import pandas as pd
import statsmodels.api as sm

try:
    import gwpy.time, gwpy.timeseries, gwpy.plotter
    import gwpy.segments
    from gwpy.timeseries import StateVector
except:
    print "gwpy import fails... no plotting possible."

__author__ = "Michael Coughlin <michael.coughlin@ligo.org>"
__version__ = 1.0
__date__    = "9/22/2013"

# =============================================================================
#
#                               DEFINITIONS
#
# =============================================================================

def parse_commandline():
    """@parse the options given on the command-line.
    """
    parser = optparse.OptionParser(usage=__doc__,version=__version__)

    parser.add_option("-i", "--inputFileDirectory", help="Seismon files.",
                      default ="/home/mcoughlin/Seismon/Text_Files/Timeseries/H0_PEM-LVEA_SEISZ/64/")

    parser.add_option("-a", "--accelerationFileDirectory", help="Seismon acceleration files.",
                      default ="/home/mcoughlin/Seismon/Text_Files/Acceleration/H0_PEM-LVEA_SEISZ/64/")
    parser.add_option("-d", "--displacementFileDirectory", help="Seismon displacement files.")
    parser.add_option("-p", "--predictionFile", help="Seismon prediction file.",
                      default ="/home/mcoughlin/Seismon/H1/H1S6/931035615-971654415/earthquakes/earthquakes.txt")
    parser.add_option("-l", "--lockFile", help="Lock segment file.",
                      default ="/home/mcoughlin/seismon/RfPrediction/data/segs_Locked_H_1126569617_1136649617.txt")
    parser.add_option("-c", "--lockChannel", help="Lock channel.",
                      default ="L1:GRD-ISC_LOCK_STATE_N")
    parser.add_option("-o", "--outputDirectory", help="output file.",
                      default ="/home/mcoughlin/Seismon/Predictions/H1S6/")

    parser.add_option("-v", "--verbose", action="store_true", default=False,
                      help="Run verbosely. (Default: False)")

    opts, args = parser.parse_args()

    # show parameters
    if opts.verbose:
        print >> sys.stderr, ""
        print >> sys.stderr, "running network_eqmon..."
        print >> sys.stderr, "version: %s"%__version__
        print >> sys.stderr, ""
        print >> sys.stderr, "***************** PARAMETERS ********************"
        for o in opts.__dict__.items():
          print >> sys.stderr, o[0]+":"
          print >> sys.stderr, o[1]
        print >> sys.stderr, ""

    return opts

def params_struct(opts):
    """@create params structure

    @param opts
        command line options
    """

    params = {}
    params["inputFileDirectory"] = opts.inputFileDirectory
    params["accelerationFileDirectory"] = opts.accelerationFileDirectory
    if not opts.displacementFileDirectory == None:
        params["displacementFileDirectory"] = opts.displacementFileDirectory
    else:
        params["displacementFileDirectory"] = params["accelerationFileDirectory"].replace("Acceleration","Displacement")
    params["predictionFile"] = opts.predictionFile
    params["lockFile"] = opts.lockFile
    params["outputDirectory"] = opts.outputDirectory

    return params

def myprior(cube, ndim, nparams):
    cube[0] = cube[0]*1.0
    cube[1] = cube[1]*2.0
    cube[2] = cube[2]*5000.0
    cube[3] = cube[3]*2.0
    return cube  

def predict_events():
    """@write pdl events

    @param params
        seismon params structure
    """

    warnings.filterwarnings("ignore")

    # Parse command line
    opts = parse_commandline()
    params = params_struct(opts)

    thresh = 1.0e-7

    if not os.path.isdir(params["outputDirectory"]):
        utils.mkdir(params["outputDirectory"])
    outputFile = os.path.join(params["outputDirectory"],"earthquakes.txt")

    fid = open(outputFile,'w')
    events = np.loadtxt(params["predictionFile"])
    neqs, ncols = events.shape

    for event in events:
        gpsMin = event[9]
        gpsMax = event[10]
   
        filename = os.path.join(params["inputFileDirectory"],"%d-%d.txt"%(gpsMin,gpsMax))

        if not os.path.isfile(filename):
            continue
        data_out_vel = np.loadtxt(filename)

        filename = os.path.join(params["accelerationFileDirectory"],"%d-%d.txt"%(gpsMin,gpsMax))
        if not os.path.isfile(filename):
            data_out_acc = -1*np.ones(data_out_vel.shape)
        else:
            data_out_acc = np.loadtxt(filename)

        filename = os.path.join(params["displacementFileDirectory"],"%d-%d.txt"%(gpsMin,gpsMax))
        if not os.path.isfile(filename):
            data_out_disp = -1*np.ones(data_out_vel.shape)
        else:
            data_out_disp = np.loadtxt(filename)

        if ncols == 15:
            fid.write("%.1f %.1f %.1f %.1f %.1f %.1f %.1f %.5e %d %d %.1f %.1f %e %.1f %.1f nan nan nan nan nan nan nan nan nan %.1f %.5e %.1f %.5e %.1f %.5e\n"%(event[0],event[1],event[2],event[3],event[4],event[5],event[6],event[7],event[8],event[9],event[10],event[11],event[12],event[13],event[14],data_out_vel[1,0],data_out_vel[1,1],data_out_acc[1,0],data_out_acc[1,1],data_out_disp[1,0],data_out_disp[1,1]))
        elif ncols == 27:
            fid.write("%.1f %.1f %.1f %.1f %.1f %.1f %.1f %.5e %d %d %.1f %.1f %e %.1f %.1f %.1f %.1f %.1f %.1f %.1f %.1f %.1f %.1f %.1f %.1f %.5e %.1f %.5e %.1f %.5e\n"%(event[0],event[1],event[2],event[3],event[4],event[5],event[6],event[7],event[8],event[9],event[10],event[11],event[12],event[13],event[14],event[15],event[16],event[17],event[18],event[19],event[20],event[21],event[22],event[23],data_out_vel[1,0],data_out_vel[1,1],data_out_acc[1,0],data_out_acc[1,1],data_out_disp[1,0],data_out_disp[1,1]))
        else:
            print("Incorrect number of columns")
            exit(0)

        print gpsMin, gpsMax

    fid.close()

    events = np.loadtxt(outputFile)
    idx = np.where(events[:,25] >= thresh)[0]
    events = events[idx,:]
    events = events[events[:,7].argsort()[::-1]]

    indexes = []
    for ii, event in enumerate(events):

        eqStart = event[2] 
        eqEnd = event[4]
        eqPeakAmp = event[-1]

        over = 0
        for jj, index in enumerate(indexes):
            event2 = events[index,:]
            eqStart2 = event2[2] 
            eqEnd2 = event2[4]

            range1 = np.arange(np.floor(eqStart),np.ceil(eqEnd))
            range2 = np.arange(np.floor(eqStart2),np.ceil(eqEnd2))

            if np.sum(np.intersect1d(range1,range2)) > 0:
                over = 1

        if over == 0:
            indexes.append(ii) 

    events = events[indexes,:]
    events = events[events[:,7].argsort()]

    total_locks = 0
    total_time = 0
    flags = []
    locklosstimes = []
    flagsls = []
    locklosstimesls = []

    segments = np.loadtxt(params["lockFile"])
    for ii, event in enumerate(events):

        eqStart = event[2]
        eqEnd = event[4]
        eqPeakAmp = event[-1]

        #if not eqStart == 1131726420.1: continue

        indexes = []
        for jj, segment in enumerate(segments):
            segStart = segment[0]
            segEnd = segment[1]

            range1 = np.arange(np.floor(eqStart),np.ceil(eqEnd))
            range2 = np.arange(np.floor(segStart),np.ceil(segEnd))

            if np.sum(np.intersect1d(range1,range2)) > 0:
                indexes.append(jj)

        locklosstime = -1
        if len(indexes) == 0:
            flag = 0
        else:
            segs = segments[indexes,:]
            checkloss = np.where(segs[:,1] <= eqEnd)[0]
            if len(checkloss) == 0:
                flag = 1
            else:
                flag = 2
                locklosstime = segs[checkloss[0],1]

        if flag == 2:
            total_time = total_time + segments[indexes[checkloss[0]]+1,1] - segments[indexes[checkloss[0]],1]
            total_locks = total_locks + 1
   
        locklosstimes.append(locklosstime)
        flags.append(flag)
 
    locklosstimes = np.array(locklosstimes)
    flags = np.array(flags)

    locklosstimes = locklosstimes[events[:,7].argsort()]
    flags = flags[events[:,7].argsort()]
    events = events[events[:,7].argsort()]
    
    fid = open(outputFile,'w')
    for ii,event in enumerate(events):
        fid.write("%.1f %.1f %.1f %.1f %.1f %.1f %.1f %.5e %d %d %.1f %.1f %e %.1f %.1f %.1f %.1f %.1f %.1f %.1f %.1f %.1f %.1f %.1f %.1f %.5e %.1f %.5e %.1f %.5e %.1f %d\n"%(event[0],event[1],event[2],event[3],event[4],event[5],event[6],event[7],event[8],event[9],event[10],event[11],event[12],event[13],event[14],event[15],event[16],event[17],event[18],event[19],event[20],event[21],event[22],event[23],event[24],event[25],event[26],event[27],event[28],event[29],locklosstimes[ii],flags[ii]))

    fid.close()
    events = np.loadtxt(outputFile)

    thresh = 1.0e-6
    idx = np.where(events[:,25] > thresh)[0]
    indexes1 = np.where(events[idx,31] == 0)[0]
    indexes2 = np.where(events[idx,31] == 1)[0]
    indexes3 = np.where(events[idx,31] == 2)[0]

    nolock = float(len(indexes1))
    nolockloss = float(len(indexes2))
    lockloss = float(len(indexes3))

    toteqs = nolockloss + lockloss + nolock
    
    print "Number of EQs above M4.5 and ground motion >= 1 micron/s: %d"%toteqs
    print "Lockloss: %.3f"%(lockloss/toteqs)
    print "No lockloss: %.3f"%(nolockloss/toteqs)
    print "Not locked: %.3f"%(nolock/toteqs)

    ind = np.arange(len(events[:,7]))
    vmin = 5.0
    vmax = 7.0

    plt.figure()
    ax = plt.gca()
    sc = ax.scatter(ind,events[:,25],s=20,c=events[:,1],vmin=vmin, vmax=vmax,label='Measured')
    ax.set_yscale('log')
    yerr = np.zeros((2,len(events)))
    yerr[0,:] = events[:,7]*4.0/5.0
    yerr[1,:] = events[:,7]*4.0
    plt.errorbar(ind,events[:,7],yerr=yerr,label='Predicted')
    plt.legend(loc='best',numpoints=1)
    plt.xlabel("Event number")
    plt.ylabel("Ground Velocity [m/s]")
    plt.xlim([-0.5,len(events[:,0])+0.5])
    plt.ylim([1e-8,1e-3])
    cbar = plt.colorbar(sc)
    cbar.set_label("Earthquake Magnitude")
    plt.show()
    plotName = os.path.join(params["outputDirectory"],'prediction.png')
    plt.savefig(plotName)
    plotName = os.path.join(params["outputDirectory"],'prediction.eps')
    plt.savefig(plotName)
    plotName = os.path.join(params["outputDirectory"],'prediction.pdf')
    plt.savefig(plotName)
    plt.close()

    vel = (events[:,12]/1000.0)/(events[:,24]-events[:,0])

    distances = np.linspace(0,100000,1000)
    times = distances / 3.5
    plt.figure()
    ax = plt.gca()
    ax.plot(events[:,7],vel,'kx')
    ax.plot([1e-7,1e-3],[3.5,3.5],'k--')
    #ax.set_yscale('log')
    ax.set_xscale('log')
    plt.xlabel("Ground velocity [m/s]")
    plt.ylabel("Earthquake velocity [m/s]")
    plt.xlim([1e-8,1e-3])
    plt.ylim([2,5])
    plt.show()
    plotName = os.path.join(params["outputDirectory"],'velocity.png')
    plt.savefig(plotName)
    plotName = os.path.join(params["outputDirectory"],'velocity.eps')
    plt.savefig(plotName)
    plotName = os.path.join(params["outputDirectory"],'velocity.pdf')
    plt.savefig(plotName)
    plt.close()

    vmin = -8
    vmax = -4

    plt.figure()
    ax = plt.gca()
    sc = ax.scatter(events[:,12],events[:,1],c=np.log10(events[:,25]),vmin=vmin, vmax=vmax)
    ax.set_xscale('log')
    plt.xlabel("Distance [m]")
    plt.ylabel("Earthquake Magnitude")
    cbar = plt.colorbar(sc)
    cbar.set_label("log10(Ground velocity [m/s])")
    #plt.xlim([1e-6,1e-3])
    #plt.ylim([5,9])
    plt.show()
    plotName = os.path.join(params["outputDirectory"],'mag_distance.png')
    plt.savefig(plotName)
    plotName = os.path.join(params["outputDirectory"],'mag_distance.eps')
    plt.savefig(plotName)
    plotName = os.path.join(params["outputDirectory"],'mag_distance.pdf')
    plt.savefig(plotName)
    plt.close()

    plt.figure()
    ax = plt.gca()
    sc = ax.scatter(events[:,12],events[:,14],c=np.log10(events[:,25]),vmin=vmin, vmax=vmax)
    ax.set_xscale('log')
    plt.xlabel("Distance [m]")
    plt.ylabel("Azimuth [deg]")
    cbar = plt.colorbar(sc)
    cbar.set_label("log10(Ground velocity [m/s])")
    #plt.xlim([1e-6,1e-3])
    #plt.ylim([5,9])
    plt.show()
    plotName = os.path.join(params["outputDirectory"],'azimuth_distance.png')
    plt.savefig(plotName)
    plotName = os.path.join(params["outputDirectory"],'azimuth_distance.eps')
    plt.savefig(plotName)
    plotName = os.path.join(params["outputDirectory"],'azimuth_distance.pdf')
    plt.savefig(plotName)
    plt.close()

    plt.figure()
    ax = plt.gca()
    sc = ax.scatter(events[:,14],events[:,1],c=np.log10(events[:,25]),vmin=vmin, vmax=vmax)
    plt.xlabel("Azimuth [deg]")
    plt.ylabel("Earthquake Magnitude")
    cbar = plt.colorbar(sc)
    cbar.set_label("log10(Ground velocity [m/s])")
    #plt.xlim([1e-6,1e-3])
    #plt.ylim([5,9])
    plt.show()
    plotName = os.path.join(params["outputDirectory"],'mag_azimuth.png')
    plt.savefig(plotName)
    plotName = os.path.join(params["outputDirectory"],'mag_azimuth.eps')
    plt.savefig(plotName)
    plotName = os.path.join(params["outputDirectory"],'mag_azimuth.pdf')
    plt.savefig(plotName)
    plt.close()

    plt.figure()
    ax = plt.gca()
    indexes = np.where(events[:,31] == 0)[0]
    sc = ax.scatter(events[indexes,12],events[indexes,25],s=20,c='b')
    indexes = np.where(events[:,31] == 1)[0]
    sc = ax.scatter(events[indexes,12],events[indexes,25],s=20,c='g',marker='+')
    indexes = np.where(events[:,31] == 2)[0]
    sc = ax.scatter(events[indexes,12],events[indexes,25],s=20,c='r',marker='x')
    ax.set_xscale('log')
    ax.set_yscale('log')
    plt.xlim([1e5,1e8])
    plt.ylim([7e-7,1e-3])
    plt.xlabel("Distance [m]")
    plt.ylabel("Peak ground motion [m/s]")
    plt.grid()
    plt.show()
    plotName = os.path.join(params["outputDirectory"],'lockloss_vel_distance.png')
    plt.savefig(plotName)
    plotName = os.path.join(params["outputDirectory"],'lockloss_vel_distance.eps')
    plt.savefig(plotName)
    plotName = os.path.join(params["outputDirectory"],'lockloss_vel_distance.pdf')
    plt.savefig(plotName)
    plt.close()


# =============================================================================
#
#                                    MAIN
#
# =============================================================================

if __name__=="__main__":

    predict_events()




import os, glob, optparse, shutil, warnings, pickle, math, copy, pickle, matplotlib
import matplotlib.pyplot as plt
import numpy as np
import scipy.signal, scipy.stats
from lxml import etree
import seismon.NLNM, seismon.html
import seismon.eqmon, seismon.utils

try:
    import gwpy.time, gwpy.timeseries
    import gwpy.frequencyseries, gwpy.spectrogram
    import gwpy.plotter
except:
    print("gwpy import fails... no plotting possible.")

__author__ = "Michael Coughlin <michael.coughlin@ligo.org>"
__date__ = "2012/8/26"
__version__ = "0.1"

# =============================================================================
#
#                               DEFINITIONS
#
# =============================================================================

def hilbert(params, segment):
    """@calculates hilbert transform for given segment.

    @param params
        seismon params dictionary
    @param segment
        [start,end] gps

    """

    from obspy.core.util.geodetics import gps2DistAzimuth

    ifo = seismon.utils.getIfo(params)
    ifolat,ifolon = seismon.utils.getLatLon(params)

    gpsStart = segment[0]
    gpsEnd = segment[1]

    # set the times
    duration = np.ceil(gpsEnd-gpsStart)

    dataAll = []
    print("Loading data...")

    for channel in params["channels"]:
        # make timeseries
        dataFull = seismon.utils.retrieve_timeseries(params, channel, segment)
        if dataFull == []:
            continue

        dataFull = dataFull / channel.calibration
        indexes = np.where(np.isnan(dataFull.data))[0]
        meanSamples = np.mean(np.ma.masked_array(dataFull.data,np.isnan(dataFull.data)))
        for index in indexes:
            dataFull[index] = meanSamples
        dataFull -= np.mean(dataFull.data)

        if np.mean(dataFull.data) == 0.0:
            print("data only zeroes... continuing\n")
            continue
        if len(dataFull.data) < 2*channel.samplef:
            print("timeseries too short for analysis... continuing\n")
            continue

        #cutoff = 0.01
        #dataFull = dataFull.lowpass(cutoff, amplitude=0.9, order=3, method='scipy')

        dataAll.append(dataFull)

    if len(dataAll) == 0:
        print("No data... returning")
        return

    if params["doEarthquakes"]:
        earthquakesDirectory = os.path.join(params["path"],"earthquakes")
        earthquakesXMLFile = os.path.join(earthquakesDirectory,"earthquakes.xml")
        attributeDics = seismon.utils.read_eqmons(earthquakesXMLFile)

    else:
        attributeDics = []

    print("Performing Hilbert transform")
    for attributeDic in attributeDics:

        if params["ifo"] == "IRIS":
            attributeDic = seismon.eqmon.ifotraveltimes(attributeDic, "IRIS", channel.latitude, channel.longitude)
            traveltimes = attributeDic["traveltimes"]["IRIS"]
        else:
            traveltimes = attributeDic["traveltimes"][ifo]

        doTilt = 0
        for dataFull in dataAll:
            if "_X_" in dataFull.channel.name or "E" == dataFull.channel.name[-1]:
                tsx = dataFull.data
            if "_Y_" in dataFull.channel.name or "N" == dataFull.channel.name[-1]:
                tsy = dataFull.data
            if "_Z_" in dataFull.channel.name:
                tsz = dataFull.data
            if "_RY_" in dataFull.channel.name:
                tttilt = np.array(dataFull.times)
                tstilt = dataFull.data
                doTilt = 1

        tt = np.array(dataAll[0].times)
        fs = 1.0/(tt[1]-tt[0])     
 
        if doTilt:   
            tstilt = np.interp(tt,tttilt,tstilt)

        Ptime = max(traveltimes["Ptimes"])
        Stime = max(traveltimes["Stimes"])
        Rtwotime = max(traveltimes["Rtwotimes"])
        RthreePointFivetime = max(traveltimes["RthreePointFivetimes"])
        Rfivetime = max(traveltimes["Rfivetimes"])
        distance = max(traveltimes["Distances"])

        indexes = np.intersect1d(np.where(tt >= Rfivetime)[0],np.where(tt <= Rtwotime)[0])
        indexes = np.intersect1d(np.where(tt >= tt[0])[0],np.where(tt <= tt[-1])[0])

        if len(indexes) == 0:
            continue

        indexMin = np.min(indexes)
        indexMax = np.max(indexes)
        tt = tt[indexes]
        tsx = tsx[indexes]
        tsy = tsy[indexes]
        tsz = tsz[indexes]

        if doTilt:
            tstilt = tstilt[indexes]

            cutoff_high = 0.01 # 10 MHz
            cutoff_low = 0.3
            n = 3
            worN = 16384
            B_low, A_low = scipy.signal.butter(n, cutoff_low / (fs / 2.0), btype='lowpass')
            #w_low, h_low = scipy.signal.freqz(B_low,A_low)
            w_low, h_low = scipy.signal.freqz(B_low,A_low,worN=worN)
            B_high, A_high = scipy.signal.butter(n, cutoff_high / (fs / 2.0), btype='highpass')
            w_high, h_high = scipy.signal.freqz(B_high,A_high,worN=worN)

            dataLowpass = scipy.signal.lfilter(B_low, A_low, tstilt,
                                        axis=0).view(dataFull.__class__)
            dataHighpass = scipy.signal.lfilter(B_high, A_high, tstilt,
                                        axis=0).view(dataFull.__class__)

            dataTilt = dataHighpass.view(dataHighpass.__class__)
            #dataTilt = tstilt.view(tstilt.__class__)
            dataTilt = gwpy.timeseries.TimeSeries(dataTilt)
            dataTilt.sample_rate = dataFull.sample_rate
            dataTilt.epoch = Rfivetime

        tszhilbert = scipy.signal.hilbert(tsz).imag
        tszhilbert = -tszhilbert

        dataHilbert = tszhilbert.view(tsz.__class__)
        dataHilbert = gwpy.timeseries.TimeSeries(dataHilbert)
        dataHilbert.sample_rate =  dataFull.sample_rate
        dataHilbert.epoch = Rfivetime

        distance,fwd,back = gps2DistAzimuth(attributeDic["Latitude"],attributeDic["Longitude"],ifolat,ifolon)
        xazimuth,yazimuth = seismon.utils.getAzimuth(params)

        angle1 = fwd
        rot1 = np.array([[np.cos(angle1), -np.sin(angle1)],[np.sin(angle1),np.cos(angle1)]])
        angle2 = xazimuth * (np.pi/180.0)
        rot2 = np.array([[np.cos(angle2), -np.sin(angle2)],[np.sin(angle2),np.cos(angle2)]])

        angleEQ = np.mod(angle1+angle2,2*np.pi)
        rot = np.array([[np.cos(angleEQ), -np.sin(angleEQ)],[np.sin(angleEQ),np.cos(angleEQ)]])

        twodarray = np.vstack([tsx,tsy])
        z = rot.dot(twodarray)
        tsxy = np.sum(z.T,axis=1)
     
        dataXY = tsxy.view(tsz.__class__)
        dataXY = gwpy.timeseries.TimeSeries(tsxy)
        dataXY.sample_rate = dataFull.sample_rate
        dataXY.epoch = Rfivetime

        dataHilbert = dataHilbert.resample(16)
        dataXY = dataXY.resample(16)

        if doTilt:
            dataTilt = dataTilt.resample(16)

        if params["doPlots"]:

            plotDirectory = params["path"] + "/Hilbert"
            seismon.utils.mkdir(plotDirectory)

            dataHilbert *= 1e6
            dataXY *= 1e6

            pngFile = os.path.join(plotDirectory,"%s.png"%(attributeDic["eventName"]))
            plot = gwpy.plotter.TimeSeriesPlot(figsize=[14,8])

            kwargs = {"linestyle":"-","color":"b"}
            plot.add_timeseries(dataHilbert,label="Hilbert",**kwargs)
            kwargs = {"linestyle":"-","color":"g"}
            plot.add_timeseries(dataXY,label="XY",**kwargs)
            plot.ylabel = r"Velocity [$\mu$m/s]"
            plot.add_legend(loc=1,prop={'size':10})

            plot.save(pngFile)
            plot.close()

        dataHilbert = tszhilbert.view(tsz.__class__)
        dataHilbert = gwpy.timeseries.TimeSeries(dataHilbert)
        dataHilbert.sample_rate =  dataFull.sample_rate
        dataHilbert.epoch = Rfivetime
        dataHilbert = dataHilbert.resample(16)

        angles = np.linspace(0,2*np.pi,10)
        xcorrs = []
        for angle in angles:
            rot = np.array([[np.cos(angle), -np.sin(angle)],[np.sin(angle),np.cos(angle)]])

            twodarray = np.vstack([tsx,tsy])
            z = rot.dot(twodarray)
            tsxy = np.sum(z.T,axis=1)

            dataXY = tsxy.view(tsz.__class__)
            dataXY = gwpy.timeseries.TimeSeries(tsxy)
            dataXY.sample_rate = dataFull.sample_rate
            dataXY.epoch = Rfivetime
            dataXY = dataXY.resample(16)

            xcorr,lags = seismon.utils.xcorr(dataHilbert.data,dataXY.data,maxlags=1)
            xcorrs.append(xcorr[1])
        xcorrs = np.array(xcorrs)

        angleMax = angles[np.argmax(xcorrs)]
        rot = np.array([[np.cos(angleMax), -np.sin(angleMax)],[np.sin(angleMax),np.cos(angleMax)]])

        twodarray = np.vstack([tsx,tsy])
        z = rot.dot(twodarray)
        tsxy = np.sum(z.T,axis=1)

        dataXY = tsxy.view(tsz.__class__)
        dataXY = gwpy.timeseries.TimeSeries(tsxy)
        dataXY.sample_rate = dataFull.sample_rate
        dataXY.epoch = Rfivetime
        dataXY = dataXY.resample(16)

        if params["doPlots"]:

            plotDirectory = params["path"] + "/Hilbert"
            seismon.utils.mkdir(plotDirectory)

            dataHilbert *= 1e6
            dataXY *= 1e6

            if doTilt:
                dataTilt *= 1e6 * (9.81)/(2*np.pi*0.01)

            pngFile = os.path.join(plotDirectory,"%s-max.png"%(attributeDic["eventName"]))
            plot = gwpy.plotter.TimeSeriesPlot(figsize=[14,8])

            kwargs = {"linestyle":"-","color":"b"}
            plot.add_timeseries(dataHilbert,label="Hilbert",**kwargs)
            kwargs = {"linestyle":"-","color":"g"}
            plot.add_timeseries(dataXY,label="XY",**kwargs)
            plot.ylabel = r"Velocity [$\mu$m/s]"
            plot.add_legend(loc=1,prop={'size':10})

            plot.save(pngFile)
            plot.close()

            pngFile = os.path.join(plotDirectory,"%s-rot.png"%(attributeDic["eventName"]))
            plot = gwpy.plotter.Plot(figsize=[14,8])

            kwargs = {"linestyle":"-","color":"b"}
            plot.add_line(angles,xcorrs,label="Xcorrs",**kwargs)
            ylim = [plot.ylim[0],plot.ylim[1]]
            kwargs = {"linestyle":"--","color":"r"}
            plot.add_line([angleEQ,angleEQ],ylim,label="EQ",**kwargs)
            plot.ylabel = r"XCorr"
            plot.xlabel = r"Angle [rad]"
            plot.add_legend(loc=1,prop={'size':10})

            plot.save(pngFile)
            plot.close()

        dataHilbert = dataHilbert.resample(1.0)
        dataXY = dataXY.resample(1.0)

        if doTilt:
            dataTilt = dataTilt.resample(1.0)

        dataHilbertAbs = np.absolute(dataHilbert)
        dataXYAbs = np.absolute(dataXY)
        dataRatio = dataXYAbs / dataHilbertAbs

        meanValue = np.median(np.absolute(dataRatio))
        dataXYScale = dataXY/meanValue

        dataXYScale = gwpy.timeseries.TimeSeries(dataXYScale)
        dataXYScale.sample_rate = dataXY.sample_rate
        dataXYScale.epoch = dataXY.epoch

        if params["doPlots"]:

            plotDirectory = params["path"] + "/Hilbert"
            seismon.utils.mkdir(plotDirectory)

            pngFile = os.path.join(plotDirectory,"%s-max-abs.png"%(attributeDic["eventName"]))
            plot = gwpy.plotter.TimeSeriesPlot(figsize=[14,8])

            kwargs = {"linestyle":"-","color":"b"}
            plot.add_timeseries(dataHilbertAbs,label="Hilbert",**kwargs)
            kwargs = {"linestyle":"-","color":"g"}
            plot.add_timeseries(dataXYAbs,label="XY",**kwargs)
            plot.ylabel = r"Velocity [$\mu$m/s]"
            plot.add_legend(loc=1,prop={'size':10})

            plot.save(pngFile)
            plot.close()

            pngFile = os.path.join(plotDirectory,"%s-max-scale.png"%(attributeDic["eventName"]))
            plot = gwpy.plotter.TimeSeriesPlot(figsize=[14,8])

            kwargs = {"linestyle":"-","color":"b"}
            plot.add_timeseries(dataHilbert,label="Hilbert",**kwargs)
            kwargs = {"linestyle":"-","color":"g"}
            plot.add_timeseries(dataXYScale,label="XY Scaled",**kwargs)
            plot.ylabel = r"Velocity [$\mu$m/s]"
            plot.add_legend(loc=1,prop={'size':10})

            plot.save(pngFile)
            plot.close()

            pngFile = os.path.join(plotDirectory,"%s-ratio.png"%(attributeDic["eventName"]))
            plot = gwpy.plotter.TimeSeriesPlot(figsize=[14,8])

            kwargs = {"linestyle":"-","color":"k"}
            plot.add_timeseries(np.absolute(dataRatio),label="Ratio",**kwargs)
            plot.ylabel = r"Ratio [XY / Hilbert(Z)]"
          
            xlim = [plot.xlim[0],plot.xlim[1]]
            kwargs = {"linestyle":"-","color":"r"}
            plot.add_line(xlim,[0.78,0.78],label="PREM",**kwargs)

            plot.title = "Median Ratio: %.3f"%(meanValue)

            #plot.ylim = [0,100]

            plot.axes[0].set_yscale("log")
            plot.add_legend(loc=1,prop={'size':10})

            plot.save(pngFile)
            plot.close()

            if doTilt:
                pngFile = os.path.join(plotDirectory,"%s-max-scale-tilt.png"%(attributeDic["eventName"]))
                plot = gwpy.plotter.TimeSeriesPlot(figsize=[14,8])

                kwargs = {"linestyle":"-","color":"b"}
                plot.add_timeseries(dataHilbert,label="Hilbert",**kwargs)
                kwargs = {"linestyle":"-","color":"g"}
                plot.add_timeseries(dataXYScale,label="XY Scaled",**kwargs)
                kwargs = {"linestyle":"-","color":"k"}
                plot.add_timeseries(dataTilt,label="Tiltmeter",**kwargs)
                plot.ylabel = r"Velocity [$\mu$m/s]"
                plot.add_legend(loc=1,prop={'size':10})

                plot.save(pngFile)
                plot.close()




#!/usr/bin/python

import os, glob, optparse, shutil, warnings, pickle, math, copy, pickle, matplotlib
import matplotlib.pyplot as plt
import numpy as np
import scipy.signal, scipy.stats
from lxml import etree
import seismon.NLNM, seismon.html
import seismon.eqmon, seismon.utils

try:
    import gwpy.time, gwpy.timeseries
    import gwpy.frequencyseries, gwpy.spectrogram
except:
    print("gwpy import fails... no plotting possible.")

__author__ = "Michael Coughlin <michael.coughlin@ligo.org>"
__date__ = "2012/8/26"
__version__ = "0.1"

# =============================================================================
#
#                               DEFINITIONS
#
# =============================================================================

def coherence(params, channel1, channel2, segment):
    """@calculates spectral data for given channel and segment.

    @param params
        seismon params dictionary
    @param channel1
        seismon channel structure
    @param channel2
        seismon channel structure
    @param segment
        [start,end] gps
    """

    ifo = seismon.utils.getIfo(params)

    gpsStart = segment[0]
    gpsEnd = segment[1]

    # set the times
    duration = np.ceil(gpsEnd-gpsStart)

    # make timeseries
    dataFull1 = seismon.utils.retrieve_timeseries(params, channel1, segment)
    if dataFull1 == []:
        return

    dataFull1 = dataFull1 / channel1.calibration
    indexes = np.where(np.isnan(dataFull1.data))[0]
    meanSamples = np.mean(np.ma.masked_array(dataFull1.data,np.isnan(dataFull1.data)))
    for index in indexes:
        dataFull1[index] = meanSamples
    dataFull1 -= np.mean(dataFull1.data)

    if np.mean(dataFull1.data) == 0.0:
        print("data only zeroes... continuing\n")
        return
    if len(dataFull1.data) < 2*channel1.samplef:
        print("timeseries too short for analysis... continuing\n")
        return

    dataFull2 = seismon.utils.retrieve_timeseries(params, channel2, segment)
    if dataFull2 == []:
        return

    dataFull2 = dataFull2 / channel2.calibration
    indexes = np.where(np.isnan(dataFull2.data))[0]
    meanSamples = np.mean(np.ma.masked_array(dataFull2.data,np.isnan(dataFull2.data)))
    for index in indexes:
        dataFull2[index] = meanSamples
    dataFull2 -= np.mean(dataFull2.data)

    if np.mean(dataFull2.data) == 0.0:
        print("data only zeroes... continuing\n")
        return
    if len(dataFull2.data) < 2*channel2.samplef:
        print("timeseries too short for analysis... continuing\n")
        return

    gpss = np.arange(gpsStart,gpsEnd,params["fftDuration"])
    fft1 = []
    fft2 = []
    for i in range(len(gpss)-1):

        tt1 = np.array(dataFull1.times)
        indexes = np.intersect1d(np.where(tt1 >= gpss[i])[0],np.where(tt1 <= gpss[i+1])[0])
        indexMin = np.min(indexes)
        indexMax = np.max(indexes)
        dataCut1 = dataFull1[indexMin:indexMax]

        dataFFT1 = dataCut1.fft(nfft=len(dataCut1))
        freqFFT1 = np.array(dataFFT1.frequencies)
        dataFFT1 = np.array(dataFFT1.data)
        indexes = np.where((freqFFT1 >= params["fmin"]) & (freqFFT1 <= params["fmax"]))[0]
        freqFFT1 = freqFFT1[indexes]
        dataFFT1 = dataFFT1[indexes]
        dataFFT1 = gwpy.frequencyseries.Spectrum(dataFFT1, f0=np.min(freqFFT1), df=(freqFFT1[1]-freqFFT1[0]))

        tt2 = np.array(dataFull2.times)
        indexes = np.intersect1d(np.where(tt2 >= gpss[i])[0],np.where(tt2 <= gpss[i+1])[0])
        indexMin = np.min(indexes)
        indexMax = np.max(indexes)
        dataCut2 = dataFull2[indexMin:indexMax]

        dataFFT2 = dataCut2.fft(nfft=len(dataCut2))
        freqFFT2 = np.array(dataFFT2.frequencies)
        dataFFT2 = np.array(dataFFT2.data)
        indexes = np.where((freqFFT2 >= params["fmin"]) & (freqFFT2 <= params["fmax"]))[0]
        freqFFT2 = freqFFT2[indexes]
        dataFFT2 = dataFFT2[indexes]
        dataFFT2 = gwpy.frequencyseries.Spectrum(dataFFT2, f0=np.min(freqFFT2), df=(freqFFT2[2]-freqFFT2[0]))

        fft1.append(dataFFT1)
        fft2.append(dataFFT2)

    specgram1 = gwpy.spectrogram.Spectrogram.from_spectra(*fft1, dt=params["fftDuration"],epoch=gpsStart)
    specgram2 = gwpy.spectrogram.Spectrogram.from_spectra(*fft2, dt=params["fftDuration"],epoch=gpsStart)

    freq = np.array(specgram1.frequencies)
    coherence = []
    coherence_phase = []
    for i in range(len(freq)):
        a1 = specgram1.data[:,i]
        psd1 = np.mean(a1 * np.conjugate(a1)).real
        a2 = specgram2.data[:,i]
        psd2 = np.mean(a2 * np.conjugate(a2)).real
        csd12 = np.mean(a1 * np.conjugate(a2))
        coh = np.absolute(csd12) / np.sqrt(psd1 * psd2)
        phase = np.angle(csd12)
        coherence.append(coh)
        coherence_phase.append(phase)
    coherence = np.array(coherence)
    coherence_phase = np.array(coherence_phase)

    coherenceDirectory = params["dirPath"] + "/Text_Files/Coherence/" + channel1.station_underscore + "_" + channel2.station_underscore + "/" + str(params["fftDuration"])
    seismon.utils.mkdir(coherenceDirectory)

    psdFile = os.path.join(coherenceDirectory,"%d-%d.txt"%(gpsStart,gpsEnd))
    f = open(psdFile,"wb")
    for i in range(len(freq)):
        f.write("%e %e\n"%(freq[i],coherence[i]))
    f.close()

    if params["doPlots"]:

        plotDirectory = params["path"] + "/" + channel1.station_underscore + "_" + channel2.station_underscore
        seismon.utils.mkdir(plotDirectory)

        pngFile = os.path.join(plotDirectory,"coh.png")

        plot = gwpy.plotter.Plot(figsize=[14,8])
        kwargs = {"linestyle":"-","color":"b"}
        plot.add_line(freq,coherence,**kwargs)
        plot.xlim = [params["fmin"],params["fmax"]]
        plot.ylim = [0,1]
        plot.xlabel = "Frequency [Hz]"
        plot.ylabel = "Coherence"

        plot.axes[0].set_yscale("log")
        if np.log10(params["fmax"]) - np.log10(params["fmin"]) > 1:
            plot.axes[0].set_xscale("log")

        plot.ylim = [np.min(coherence),1]
        plot.save(pngFile,dpi=200)
        plot.close()

        pngFile = os.path.join(plotDirectory,"coh_phase.png")

        plot = gwpy.plotter.Plot(figsize=[14,8])
        kwargs = {"linestyle":"-","color":"b"}
        plot.add_line(freq,coherence_phase,**kwargs)
        plot.xlim = [params["fmin"],params["fmax"]]
        #plot.ylim = [0,1]
        plot.xlabel = "Frequency [Hz]"
        plot.ylabel = "Coherence Phase"

        if np.log10(params["fmax"]) - np.log10(params["fmin"]) > 1:
            plot.axes[0].set_xscale("log")

        plot.save(pngFile,dpi=200)
        plot.close()


def coherence_summary(params, channel1, segment):
    """@summary of channels of spectral data.

    @param params
        seismon params dictionary
    @param channel1
        seismon channel structure
    @param segment
        [start,end] gps
    """

    gpsStart = segment[0]
    gpsEnd = segment[1]

    data = {}
    for channel2 in params["channels"]:

        coherenceDirectory = params["dirPath"] + "/Text_Files/Coherence/" + channel1.station_underscore + "_" + channel2.station_underscore + "/" + str(params["fftDuration"])
        file = os.path.join(coherenceDirectory,"%d-%d.txt"%(gpsStart,gpsEnd))

        if not os.path.isfile(file):
            continue

        spectra_out = gwpy.frequencyseries.Spectrum.read(file)
        spectra_out.unit = 'counts/Hz^(1/2)'

        if np.sum(spectra_out.data) == 0.0:
            continue

        data[channel2.station_underscore] = {}
        data[channel2.station_underscore]["data"] = spectra_out

    if data == {}:
        return

    if params["doPlots"]:

        plotDirectory = params["path"] + "/coherence_summary"
        seismon.utils.mkdir(plotDirectory)

        pngFile = os.path.join(plotDirectory,"%s.png"%(channel1.station_underscore))
        lowBin = np.inf
        highBin = -np.inf
        plot = gwpy.plotter.Plot(figsize=[14,8])
        for key in data.iterkeys():

            label = key.replace("_","\_")

            plot.add_spectrum(data[key]["data"], label=label)
            lowBin = np.min([lowBin,np.min(data[key]["data"])])
            highBin = np.max([highBin,np.max(data[key]["data"])])

        plot.xlim = [params["fmin"],params["fmax"]]
        plot.ylim = [lowBin, highBin]
        plot.xlabel = "Frequency [Hz]"
        plot.ylabel = "Coherence"
        plot.add_legend(loc=1,prop={'size':10})

        plot.axes[0].set_yscale("log") 
        if np.log10(params["fmax"]) - np.log10(params["fmin"]) > 1:
            plot.axes[0].set_xscale("log")

        plot.save(pngFile,dpi=200)
        plot.close()





# Copyright (C) 2013 Michael Coughlin
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""Seismic summary information generator.

This script generates a nested web page and content for
reviewing seismicity at the sites and earthquake status.

Comments should be e-mailed to michael.coughlin@ligo.org.

"""


import os, sys, glob, optparse, shutil, warnings

if not os.getenv("DISPLAY", None):
    import matplotlib
    matplotlib.use("agg", warn=False)

import lal.gpstime

import seismon.psd, seismon.utils
import seismon.html, seismon.omicron
import seismon.eqmon, seismon.eqmon_plot
import seismon.coherence, seismon.hilbert
import seismon.wiener, seismon.wiener_fft
import seismon.trend, seismon.frame
import seismon.bits

__author__ = "Michael Coughlin <michael.coughlin@ligo.org>"
__version__ = 1.0
__date__    = "9/22/2013"

# =============================================================================
#
#                               DEFINITIONS
#
# =============================================================================

def parse_commandline():
    """@Parse the options given on the command-line.
    """
    parser = optparse.OptionParser(usage=__doc__,version=__version__)

    parser.add_option("-p", "--paramsFile", help="Seismon params file.", 
                      default ="/home/mcoughlin/Seismon/seismon/input/seismon_params_HIFOY.txt")

    parser.add_option("-s", "--gpsStart", help="GPS Start Time.", default=1054028959,type=int)
    parser.add_option("-e", "--gpsEnd", help="GPS End Time.", default=1054029209,type=int)
    parser.add_option("-f", "--fftDuration", help="FFT duration.", default=64,type=int)

    parser.add_option("-c", "--channel", help="Run on a specific channel.", default=None)
    parser.add_option("-r", "--referenceChannel", help="Channel for PSD comparison.", default=None)

    parser.add_option("-m", "--minMagnitude", help="Minimum earthquake magnitude.", default=5.0,type=float)

    parser.add_option("--gpsSpectraStart", help="GPS spectra Start Time.", type=int)
    parser.add_option("--gpsSpectraEnd", help="GPS spectra End Time.", type=int)

    parser.add_option("--segmentDatabase", help="Segment database", default="https://segdb2.ligo.caltech.edu")
    parser.add_option("--segmentFlag", help="Segment flag", default="H1:ODC-PSL_SUMMARY:1")
    parser.add_option("--segmentsTextFile", help="Segments text file.", default=None)

    parser.add_option("--flagsDatabase", help="Flags database", default="https://segdb.ligo.caltech.edu")
    parser.add_option("--flagsFlag", help="Flags flag", default="H1:DMT-BRMS_SEISMIC_Y_30_100_MHZ_HIGHTHRESH")
    parser.add_option("--flagsTextFile", help="Flags text file.", 
                     default="/home/mcoughlin/Seismon/seismon/input/seismon_flags_BLRMS.txt")
    parser.add_option("--flagsChannelTextFile", help="Flags text file.",
                     default="/home/mcoughlin/Seismon/seismon/input/seismon_flags_BLRMS.txt")

    parser.add_option("--fmin", help="fmin.", default=1.0/64.0,type=float)
    parser.add_option("--fmax", help="fmax.", default=64.0,type=float)

    parser.add_option("--eventfilesType", help="Event files type.", default="public")
    parser.add_option("--userEmail", help="User e-mail.", default="public")

    parser.add_option("--doPlots",  action="store_true", default=False)
    parser.add_option("--doEarthquakes",  action="store_true", default=False)
    parser.add_option("--doEarthquakesAnalysis",  action="store_true", default=False)
    parser.add_option("--doEarthquakesOnline",  action="store_true", default=False)
    parser.add_option("--doEarthquakesPicks",  action="store_true", default=False)
    parser.add_option("--doEarthquakesVelocityMap",  action="store_true", default=False)
    parser.add_option("--doEarthquakesLookUp",  action="store_true", default=False)
    parser.add_option("--doEarthquakesTraining",  action="store_true", default=False)
    parser.add_option("--doEarthquakesTesting",  action="store_true", default=False)
    parser.add_option("--doEarthquakesHilbert",  action="store_true", default=False)
    parser.add_option("--doEarthquakesChile",  action="store_true", default=False)
    parser.add_option("--earthquakesTrainingFile", help="Earthquakes training file.",
                     default="/home/mcoughlin/Seismon/H1/H1S5-815097613-875145614/earthquakes_training/training.xml")
    parser.add_option("--doEarthquakesTrips",  action="store_true", default=False)
    parser.add_option("--tripsTextFile", help="Flags text file.",
                     default="/home/mcoughlin/Seismon/Trips/H1_trips.txt")

    parser.add_option("--doOmicron",  action="store_true", default=False)
    parser.add_option("--doFreqAnalysis",  action="store_true", default=False)
    parser.add_option("--doAnalysis",  action="store_true", default=False)
    parser.add_option("--doHTML",  action="store_true", default=False)
    parser.add_option("--doSegmentsDatabase",  action="store_true", default=False)
    parser.add_option("--doSegmentsTextFile",  action="store_true", default=False)
    parser.add_option("--doPSD",  action="store_true", default=False)    
    parser.add_option("--doBits",  action="store_true", default=False)
    parser.add_option("--doTrend",  action="store_true", default=False)
    parser.add_option("--doSummary",  action="store_true", default=False)
    parser.add_option("--doCoherence",  action="store_true", default=False)
    parser.add_option("--doCoherenceSummary",  action="store_true", default=False)
    parser.add_option("--doBeamForming",  action="store_true", default=False)
    parser.add_option("--doStrainz",  action="store_true", default=False)
    parser.add_option("--doWiener",  action="store_true", default=False)
    parser.add_option("--doWienerFFT",  action="store_true", default=False)
    parser.add_option("--doWienerHilbert",  action="store_true", default=False)
    parser.add_option("--doWienerSummary",  action="store_true", default=False)
    parser.add_option("--noFrames",  action="store_true", default=False)
    parser.add_option("--doMakeFrames",  action="store_true", default=False)
    parser.add_option("--doMakeFramesCalibrated",  action="store_true", default=False)
    parser.add_option("--framesSampleRate", help="frames sample rate.", default='0')
    parser.add_option("--framesFolder", help="frames folder.", 
                     default="/home/mcoughlin/Gravimeter/frames")
    parser.add_option("--framesFolderCalibrated", help="frames folder.",
                     default="/home/mcoughlin/Gravimeter/frames_calibrated")
    parser.add_option("--doPowerLawFit",  action="store_true", default=False)

    parser.add_option("-N", "--wienerFilterOrder", help="Wiener filter order.", default=1000,type=int)
    parser.add_option("--wienerFilterSampleRate", help="Wiener filter sample rate.", default=0,type=int)
    parser.add_option("--wienerFilterLowFreq", help="Wiener filter lowpass.", default=0,type=float)
    parser.add_option("--wienerFilterHighFreq", help="Wiener filter highpass", default=0,type=float)

    parser.add_option("--doFlagsDatabase",  action="store_true", default=False)
    parser.add_option("--doFlagsTextFile",  action="store_true", default=False)
    parser.add_option("--doFlagsChannel",  action="store_true", default=False)
    parser.add_option("--doFlagsAnalysis",  action="store_true", default=False)

    parser.add_option("--doKML",  action="store_true", default=False)
    parser.add_option("--doviz",  action="store_true", default=False)

    parser.add_option("-v", "--verbose", action="store_true", default=False,
                      help="Run verbosely. (Default: False)")

    opts, args = parser.parse_args()

    # show parameters
    if opts.verbose:
        print >> sys.stderr, ""
        print >> sys.stderr, "running pylal_seismon_run..."
        print >> sys.stderr, "version: %s"%__version__
        print >> sys.stderr, ""
        print >> sys.stderr, "***************** PARAMETERS ********************"
        for o in opts.__dict__.items():
          print >> sys.stderr, o[0]+":"
          print >> sys.stderr, o[1]
        print >> sys.stderr, ""

    return opts

def params_struct(opts):
    """@Creates seismon params structure
    @param opts
        seismon command line options
    """

    params = seismon.utils.readParamsFromFile(opts.paramsFile)
    params["gpsStart"] = opts.gpsStart
    params["gpsEnd"] = opts.gpsEnd
    params["minMagnitude"] = opts.minMagnitude
    params["fftDuration"] = opts.fftDuration

    if opts.channel == None:
        params["channel"] = opts.channel
    else:
        params["channel"] = [x for x in opts.channel.split(",")]
    params["referenceChannel"] = opts.referenceChannel

    if opts.gpsSpectraStart == None:
        params["gpsSpectraStart"] = opts.gpsStart - 24*60*60
    else:
        params["gpsSpectraStart"] = opts.gpsSpectraStart

    if opts.gpsSpectraEnd == None:
        params["gpsSpectraEnd"] = opts.gpsEnd 
    else:
        params["gpsSpectraEnd"] = opts.gpsSpectraEnd

    params["segmentDatabase"] = opts.segmentDatabase
    params["segmentFlag"] = opts.segmentFlag
    params["segmentsTextFile"] = opts.segmentsTextFile
    params["flagsDatabase"] = opts.flagsDatabase
    params["flagsFlag"] = opts.flagsFlag
    params["flagsTextFile"] = opts.flagsTextFile
    params["flagsChannelTextFile"] = opts.flagsTextFile

    params["eventfilesType"] = opts.eventfilesType

    params["fmin"] = opts.fmin
    params["fmax"] = opts.fmax
    params["doPlots"] = opts.doPlots
    params["doEarthquakes"] = opts.doEarthquakes
    params["doEarthquakesAnalysis"] = opts.doEarthquakesAnalysis
    params["doEarthquakesOnline"] = opts.doEarthquakesOnline
    params["doEarthquakesPicks"] = opts.doEarthquakesPicks
    params["doEarthquakesVelocityMap"] = opts.doEarthquakesVelocityMap
    params["doEarthquakesLookUp"] = opts.doEarthquakesLookUp
    params["doEarthquakesTraining"] = opts.doEarthquakesTraining
    params["doEarthquakesTesting"] = opts.doEarthquakesTesting
    params["doEarthquakesHilbert"] = opts.doEarthquakesHilbert
    params["doEarthquakesTrips"] = opts.doEarthquakesTrips
    params["doEarthquakesChile"] = opts.doEarthquakesChile
    params["tripsTextFile"] = opts.tripsTextFile

    params["userEmail"] = opts.userEmail
    params["earthquakesTrainingFile"] = opts.earthquakesTrainingFile

    params["doOmicron"] = opts.doOmicron
    params["doAnalysis"] = opts.doAnalysis
    params["doFreqAnalysis"] = opts.doFreqAnalysis
    params["doHTML"] = opts.doHTML
    params["doSegmentsDatabase"] = opts.doSegmentsDatabase
    params["doSegmentsTextFile"] = opts.doSegmentsTextFile
    params["doPSD"] = opts.doPSD
    params["doBits"] = opts.doBits
    params["doTrend"] = opts.doTrend
    params["doSummary"] = opts.doSummary
    params["doCoherence"] = opts.doCoherence
    params["doCoherenceSummary"] = opts.doCoherenceSummary
    params["doBeamForming"] = opts.doBeamForming
    params["doStrainz"] = opts.doStrainz
    params["doWiener"] = opts.doWiener
    params["doWienerFFT"] = opts.doWienerFFT
    params["doWienerHilbert"] = opts.doWienerHilbert
    params["doWienerSummary"] = opts.doWienerSummary
    params["wienerFilterOrder"] = opts.wienerFilterOrder
    params["wienerFilterSampleRate"] = opts.wienerFilterSampleRate
    params["wienerFilterLowFreq"] = opts.wienerFilterLowFreq
    params["wienerFilterHighFreq"] = opts.wienerFilterHighFreq
    params["noFrames"] = opts.noFrames
    params["doMakeFrames"] = opts.doMakeFrames
    params["doMakeFramesCalibrated"] = opts.doMakeFramesCalibrated
    params["framesSampleRate"] = float(eval(opts.framesSampleRate))
    params["framesFolder"] = opts.framesFolder
    params["framesFolderCalibrated"] = opts.framesFolderCalibrated
    params["doPowerLawFit"] = opts.doPowerLawFit

    params["doFlagsDatabase"] = opts.doFlagsDatabase
    params["doFlagsTextFile"] = opts.doFlagsTextFile
    params["doFlagsChannel"] = opts.doFlagsChannel
    params["doFlagsAnalysis"] = opts.doFlagsAnalysis

    params["doKML"] = opts.doKML
    params["doviz"] = opts.doviz

    if params["gpsStart"] == 1025136015:
        params["gpsStart"] = params["gpsStart"] + 1

    params["date"] = lal.gpstime.gps_to_utc(params["gpsStart"])
    #params["date"] = XLALGPSToUTC(LIGOTimeGPS(params["gpsStart"]))
    params["dateString"] = params["date"].strftime("%Y-%m-%d %H:%M:%S")
    #params["dateString"] = "%d-%d-%d %d:%d:%d"%(params["date"][0],params["date"][1],params["date"][2],params["date"][3],params["date"][4],params["date"][5])

    if params["doEarthquakesAnalysis"]:
       params["earthquakesMinMag"] = params["minMagnitude"]
    else:
       params["earthquakesMinMag"] = 0

    if params["ifo"] == "IRIS":
        import obspy.iris, obspy.core, obspy.fdsn
        params["client"] = obspy.fdsn.client.Client("IRIS")

    return params

# =============================================================================
#
#                                    MAIN
#
# =============================================================================

warnings.filterwarnings("ignore")

# Parse command line
opts = parse_commandline()
params = params_struct(opts)

if params["doEarthquakesAnalysis"]:
    channelList = params["codePath"] + "/seismon/input/seismon-" + params["ifo"] + "-" + params["frameType"] + "-channel_list.txt"
else:
    channelList = params["codePath"] + "/seismon/input/seismon-" + params["ifo"] + "-" + params["frameType"] + "-channel_list.txt"

params = seismon.utils.channel_struct(params,channelList)
params = seismon.utils.segment_struct(params)
params = seismon.utils.frame_struct(params)

if params["frame"] == [] and not params["noFrames"]:
    print "No frames found... quitting."
    sys.exit()

params = seismon.utils.setPath(params,[params["gpsStart"],params["gpsEnd"]])

print params["path"]

if params["doEarthquakesOnline"]:
    segmentlist = seismon.eqmon.run_earthquakes(params,[params["gpsStart"],params["gpsEnd"]])
    sys.exit()
if params["doEarthquakesAnalysis"]:
    params["segments"] = seismon.eqmon.run_earthquakes(params,[params["gpsStart"],params["gpsEnd"]])

for segment in params["segments"]:
    params = seismon.utils.setPath(params,segment)
    params = seismon.utils.flag_struct(params,segment)

if params["doEarthquakes"]:
    print "Finding earthquakes"
    for segment in params["segments"]:
        params = seismon.utils.setPath(params,segment)
        segments = seismon.eqmon.run_earthquakes(params,segment)

if params["doOmicron"]:
    print "Generating Omicron triggers"
    #seismon.omicron.generate_triggers(params)

if params["doviz"]:

    import seismon.viz

    for channel in params["channels"]:
        print "Generating bokeh page for %s"%channel.station
        seismon.viz.channel_page(params,channel)
    sys.exit()

for channel in params["channels"]:
    if params["doPSD"]:
        print "Generating PSD for %s"%channel.station
        for segment in params["segments"]:
            print "Segment: %d-%d"%(segment[0],segment[1])
            params = seismon.utils.setPath(params,segment)

            timeseriesDirectory = params["dirPath"] + "/Text_Files/Timeseries/" + channel.station_underscore + "/" + str(params["fftDuration"])
            gpsStart = segment[0]
            gpsEnd = segment[1]
            timeseriesFile = os.path.join(timeseriesDirectory,"%d-%d.txt"%(gpsStart,gpsEnd))
            if os.path.isfile(timeseriesFile): continue

            seismon.psd.spectra(params,channel,segment)
    if params["doAnalysis"]:
        print "Analyzing PSD significance for %s"%channel.station
        seismon.psd.analysis(params,channel)
    if params["doOmicron"]:
        for segment in params["segments"]:
            print "Plotting Omicron triggers for %s"%channel.station
            params = seismon.utils.setPath(params,segment)
            seismon.omicron.plot_triggers(params,channel,segment)
    if params["doTrend"]:
        print "Generating trend for %s"%channel.station
        for segment in params["segments"]:
            print "Segment: %d-%d"%(segment[0],segment[1])
            params = seismon.utils.setPath(params,segment)
            seismon.trend.trend(params,channel,segment)
    if params["doBits"]:
        print "Generating bits for %s"%channel.station
        for segment in params["segments"]:
            print "Segment: %d-%d"%(segment[0],segment[1])
            params = seismon.utils.setPath(params,segment)
            seismon.bits.bits(params,channel,segment)   
 
if params["doCoherence"]:
    for i in xrange(len(params["channels"])):
        channel1 = params["channels"][i]
        for j in xrange(len(params["channels"])):
            channel2 = params["channels"][j]
            if j <= i:
                continue
            print "Generating coherence for %s, %s"%(channel1.station,channel2.station)
            for segment in params["segments"]:
                print "Segment: %d-%d"%(segment[0],segment[1])
                params = seismon.utils.setPath(params,segment)
                seismon.coherence.coherence(params,channel1,channel2,segment)

if params["doBeamForming"]:
    import seismon.beamforming
    print "Running beamforming"
    for segment in params["segments"]:
        print "Segment: %d-%d"%(segment[0],segment[1])
        params = seismon.utils.setPath(params,segment)
        seismon.beamforming.beamforming(params,segment)

if params["doStrainz"]:
    import seismon.beamforming
    print "Running strainz"
    for segment in params["segments"]:
        print "Segment: %d-%d"%(segment[0],segment[1])
        params = seismon.utils.setPath(params,segment)
        seismon.beamforming.strainz(params,segment)

if params["doEarthquakesHilbert"]:
    print "Running Hilbert transform"
    for segment in params["segments"]:
        print "Segment: %d-%d"%(segment[0],segment[1])
        params = seismon.utils.setPath(params,segment)
        seismon.hilbert.hilbert(params,segment)

if params["doSummary"]:
    for segment in params["segments"]:
        print "Running summary for Segment: %d-%d"%(segment[0],segment[1])
        params = seismon.utils.setPath(params,segment)
        seismon.psd.channel_summary(params, segment)
    print "Summaries complete!"

if params["doCoherenceSummary"]:
    for i in xrange(len(params["channels"])):
        channel = params["channels"][i]
        print "Generating coherence summary for %s"%(channel.station)
        for segment in params["segments"]:
            print "Segment: %d-%d"%(segment[0],segment[1])
            params = seismon.utils.setPath(params,segment)
            seismon.coherence.coherence_summary(params,channel,segment)

if params["doWiener"]:
    for channel in params["channels"]:
        print "Running wiener filter for %s"%(channel.station)
        for segment in params["segments"]:
            print "Segment: %d-%d"%(segment[0],segment[1])
            params = seismon.utils.setPath(params,segment)
            seismon.wiener.wiener(params,channel,segment)

if params["doWienerFFT"]:
    for channel in params["channels"]:
        print "Running wiener filter for %s"%(channel.station)
        for segment in params["segments"]:
            print "Segment: %d-%d"%(segment[0],segment[1])
            params = seismon.utils.setPath(params,segment)
            seismon.wiener_fft.wiener(params,channel,segment)

if params["doWienerHilbert"]:
    print "Running wiener / hilbert filter"
    for segment in params["segments"]:
        print "Segment: %d-%d"%(segment[0],segment[1])
        params = seismon.utils.setPath(params,segment)
        seismon.wiener.wiener_hilbert(params,segment)

if params["doWienerSummary"]:
    for channel in params["channels"]:
        print "Generating wiener summary for %s"%(channel.station)
        for segment in params["segments"]:
            print "Segment: %d-%d"%(segment[0],segment[1])
            params = seismon.utils.setPath(params,segment)
            seismon.wiener.wiener_summary(params,channel,segment)

if params["doMakeFrames"]:
    for segment in params["segments"]:
        print "Running frames for Segment: %d-%d"%(segment[0],segment[1])
        params = seismon.utils.setPath(params,segment)
        seismon.frame.make_frames(params, segment)
    print "Frames complete!"    

if params["doMakeFramesCalibrated"]:
    for segment in params["segments"]:
        print "Calibrating frames for Segment: %d-%d"%(segment[0],segment[1])
        params = seismon.utils.setPath(params,segment)
        seismon.frame.make_frames_calibrated(params, segment)
    print "Frames complete!"

if params["doEarthquakesAnalysis"]:
    print "Running earthquake analysis"
    params = seismon.utils.setPath(params,[params["gpsStart"],params["gpsEnd"]])
    seismon.eqmon.run_earthquakes_analysis(params,[params["gpsStart"],params["gpsEnd"]])

if params["doFlagsAnalysis"]:
    print "Running flags analysis"
    params = seismon.utils.setPath(params,[params["gpsStart"],params["gpsEnd"]])
    seismon.utils.run_flags_analysis(params,[params["gpsStart"],params["gpsEnd"]])

if params["doHTML"]:
    htmlPage = seismon.html.summary_page(params)
    if htmlPage is not None:
        f = open(os.path.join(params["path"],"summary.html"),"w")
        f.write(htmlPage)
        f.close()

    # Public HTML output path
    params["outputPath"] = os.path.join(params["publicPath"],params["ifo"]);
    seismon.utils.mkdir(params["outputPath"])

    #os.system("rm -r %s/%s"%(params["outputPath"],params["runName"]))
    #os.system("cp -r %s %s/%s"%(params["path"],params["outputPath"],params["runName"]))





"""
%prog

Michael Coughlin (coughlim@carleton.edu)

This program runs seismon_run.py.

"""

# =============================================================================
#
#                               PREAMBLE
#
# =============================================================================

import os, time


#stupid astropy cludge here
#os.system("mv /ligo/home/controls/.astropy/config/astropy.cfg /ligo/home/controls/.astropy/config/astropy.cfg.old")

__author__ = "Michael Coughlin <coughlim@carleton.edu>"
__date__ = "2012/2/7"
__version__ = "0.1"

gpsEnd = int(os.popen("tconvert now").readline())

while 1:
   
   #stupid astropy cludge here
   #os.system("mv /ligo/home/controls/.astropy/config/astropy.cfg /ligo/home/controls/.astropy/config/astropy.cfg.old")

   paramsFile = "/usr/local/seismon/seismon/seismon/input/seismon_params_H1.txt"
   timewindow = 10*3600
   gpsStart = int(os.popen("tconvert now").readline()) - timewindow
   gpsEnd = gpsStart + timewindow

   start_time = time.time()
   os.system("/usr/local/seismon/ve/bin/python seismon_info -p %s -s %d -e %d --eventfilesType private --doEarthquakes --doEPICs --doPurge"%(paramsFile,gpsStart,gpsEnd))
   end_time = time.time()
   elapsed_time = end_time - start_time
   print("Elapsed time for Seismon: %f"%(elapsed_time))
   print("python seismon_info -p %s -s %d -e %d --eventfilesType private --doEarthquakes --doEPICs --doPurge"%(paramsFile,gpsStart,gpsEnd))

   #start_time = time.time()
   #os.system("/usr/bin/python seismon_info -p %s -s %d -e %d --eventfilesType private --doEarthquakes --doReadEPICs --doPlots"%(paramsFile,gpsEnd-3600,gpsEnd))
   #end_time = time.time()
   #elapsed_time = end_time - start_time
   #print "Elapsed time for Seismon: %f"%(elapsed_time)
   #print "python seismon_info -p %s -s %d -e %d --eventfilesType private --doEarthquakes --doReadEPICs --doPlots"%(paramsFile,gpsEnd-3600,gpsEnd)





# Copyright (C) 2013 Michael Coughlin
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""Earthquake xml file generator.

This script generates earthquake xml files using notices from the
internet and USGS PDL client.

Comments should be e-mailed to michael.coughlin@ligo.org.

"""

import os, sys, glob, optparse, warnings, time, json
import numpy as np
from operator import itemgetter

import matplotlib
matplotlib.use("AGG")
matplotlib.rcParams.update({'font.size': 18})
from matplotlib import pyplot as plt

import pylal.pylal_seismon_eqmon, pylal.pylal_seismon_utils

from obspy.taup.taup import getTravelTimes
from obspy.core.util.geodetics import gps2DistAzimuth

import lal.gpstime

__author__ = "Michael Coughlin <michael.coughlin@ligo.org>"
__version__ = 1.0
__date__    = "9/22/2013"

# =============================================================================
#
#                               DEFINITIONS
#
# =============================================================================

def parse_commandline():
    """@parse the options given on the command-line.
    """
    parser = optparse.OptionParser(usage=__doc__,version=__version__)

    parser.add_option("-p", "--paramsFile", help="Seismon params file.",
                      default ="/home/mcoughlin/Seismon/seismon/input/seismon_params_traveltimes.txt")
    parser.add_option("-s", "--gpsStart", help="GPS Start Time.", default=0,type=int)
    parser.add_option("-e", "--gpsEnd", help="GPS End Time.", default=2000000000,type=int)
    parser.add_option("-m", "--minMagnitude", help="Minimum earthquake magnitude.", default=5.0,type=float)
    parser.add_option("-i", "--ifo", help="Ifo.", default="H1")
    parser.add_option("-v", "--verbose", action="store_true", default=False,
                      help="Run verbosely. (Default: False)")
    parser.add_option("-t","--types",  default="private,iris")

    opts, args = parser.parse_args()

    # show parameters
    if opts.verbose:
        print >> sys.stderr, ""
        print >> sys.stderr, "running network_eqmon..."
        print >> sys.stderr, "version: %s"%__version__
        print >> sys.stderr, ""
        print >> sys.stderr, "***************** PARAMETERS ********************"
        for o in opts.__dict__.items():
          print >> sys.stderr, o[0]+":"
          print >> sys.stderr, o[1]
        print >> sys.stderr, ""

    return opts

def params_struct(opts):
    """@create params structure

    @param opts
        command line options
    """

    params = pylal.pylal_seismon_utils.readParamsFromFile(opts.paramsFile)
    params["gpsStart"] = opts.gpsStart
    params["gpsEnd"] = opts.gpsEnd
    params["minMagnitude"] = opts.minMagnitude
    params["ifo"] = opts.ifo

    if opts.types == None:
        params["types"] = opts.types
    else:
        params["types"] = [x for x in opts.types.split(",")]

    return params

def histogramPlot(data,type,plotName):

    fig = plt.Figure(figsize=[14,8])
    ax = plt.subplot(111)

    hist, bins = np.histogram(data, bins=15)
    width = 0.7 * (bins[1] - bins[0])
    center = (bins[:-1] + bins[1:]) / 2
    plt.bar(center, hist, align='center', width=width)

    if type == "Magnitude":
        xlabel = "$\Delta$ Magnitude"
    elif type == "Depth":
        xlabel = "$\Delta$ Depth [km]"
    elif type == "Distance":
        xlabel = "$\Delta$ Distance [m]"
    elif type == "Rfamp":
        xlabel = "$\Delta$ Amplitude [$\mu$m/s]"
    elif type == "RfampPerc":
        xlabel = "log10(Amplitude Relative Difference)"
    elif type == "latencies":
        xlabel = "Time [s]"

    plt.xlabel(xlabel)
    plt.ylabel('Counts')

    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')


def comparison(params,type1,type2):

    params["earthquakesMinMag"] = params["minMagnitude"]
    ifo = pylal.pylal_seismon_utils.getIfo(params)

    params["eventfilesType"] = type1
    attributeDics1 = pylal.pylal_seismon_eqmon.retrieve_earthquakes(params,params["gpsStart"],params["gpsEnd"])
    attributeDics1 = sorted(attributeDics1, key=itemgetter("Magnitude"), reverse=True)

    params["eventfilesType"] = type2
    attributeDics2 = pylal.pylal_seismon_eqmon.retrieve_earthquakes(params,params["gpsStart"],params["gpsEnd"])
    attributeDics2 = sorted(attributeDics2, key=itemgetter("Magnitude"), reverse=True)

    performanceDirectory = "/home/mcoughlin/Seismon/performance"
    pylal.pylal_seismon_utils.mkdir(performanceDirectory)
        
    performanceFile = os.path.join(performanceDirectory,"%s.txt"%params["ifo"])
    f = open(performanceFile,"w")
    
    timeDifference = 10
    for attributeDic1 in attributeDics1:
        for attributeDic2 in attributeDics2:
            if np.absolute(attributeDic1["GPS"] - attributeDic2["GPS"]) <= timeDifference:

                attributeDic1 = pylal.pylal_seismon_eqmon.calculate_traveltimes(attributeDic1)
                attributeDic2 = pylal.pylal_seismon_eqmon.calculate_traveltimes(attributeDic2)

                traveltimes1 = attributeDic1["traveltimes"][ifo]
                traveltimes2 = attributeDic2["traveltimes"][ifo]

                gps1 = attributeDic1["GPS"]
                gps2 = attributeDic2["GPS"]

                sentgps1 = attributeDic1["SentGPS"]
                sentgps2 = attributeDic2["SentGPS"]

                writtengps1 = attributeDic1["WrittenGPS"]
                writtengps2 = attributeDic2["WrittenGPS"]

                if attributeDic1["SentGPS"] > traveltimes1["Ptimes"][-1]:
                    continue

                magnitude1 = attributeDic1["Magnitude"]
                magnitude2 = attributeDic2["Magnitude"]
 
                depth1 = attributeDic1["Depth"]
                depth2 = attributeDic2["Depth"]

                distance1 = max(traveltimes1["Distances"])
                distance2 = max(traveltimes2["Distances"])

                Rfamp1 = traveltimes1["Rfamp"][0] 
                Rfamp2 = traveltimes2["Rfamp"][0]

                print Rfamp1, Rfamp2

                f.write("%.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f %e %e\n"%(gps1,gps2,sentgps1,sentgps2,magnitude1,magnitude2,depth1,depth2,distance1,distance2,Rfamp1,Rfamp2))
    f.close()

    data = np.loadtxt(performanceFile)

    magnitudeDiff = data[:,5] - data[:,4]
    plotName = os.path.join(performanceDirectory,"magnitudeDiff.png")
    histogramPlot(magnitudeDiff,"Magnitude",plotName)
    depthDiff = data[:,7] - data[:,6]
    plotName = os.path.join(performanceDirectory,"depthDiff.png")
    histogramPlot(depthDiff,"Depth",plotName)
    distanceDiff = data[:,9] - data[:,8]
    plotName = os.path.join(performanceDirectory,"distanceDiff.png")
    histogramPlot(distanceDiff,"Distance",plotName)
    RfampDiff = 1e6 * (data[:,11] - data[:,10])
    plotName = os.path.join(performanceDirectory,"RfampDiff.png")
    histogramPlot(RfampDiff,"Rfamp",plotName)
    RfampDiffPerc = np.absolute(data[:,11] - data[:,10]) / np.min([data[:,11],data[:,10]])

    RfampDiffPerc = []
    for Rfamp1,Rfamp2 in zip(data[:,10],data[:,11]):
        RfampDiffPerc.append(np.absolute(Rfamp1-Rfamp2)/np.min([Rfamp1,Rfamp2]))

    RfampDiffPerc = np.log10(RfampDiffPerc)
    plotName = os.path.join(performanceDirectory,"RfampDiffPerc.png")
    histogramPlot(RfampDiffPerc,"RfampPerc",plotName)
    latencies = np.absolute(data[:,2] - data[:,0])
    plotName = os.path.join(performanceDirectory,"latencies.png")
    histogramPlot(latencies,"latencies",plotName)

def run_comparison():
    """@run traveltime calculator
    """

    warnings.filterwarnings("ignore")

    # Parse command line
    opts = parse_commandline()
    # Parse command line
    params = params_struct(opts)

    for i in xrange(len(params["types"])):
        for j in xrange(len(params["types"])):
            if i < j:
                type1 = params["types"][i]
                type2 = params["types"][j]
                comparison(params,type1,type2)
   
# =============================================================================
#
#                                    MAIN
#
# =============================================================================

if __name__=="__main__":

    run_comparison()

